################################################################################
# Define list of package names and required versions.
################################################################################
deppkgs = c(
  ape = "4.1",
  caret = "6.0.76",
  cluster = "2.0.6",
  clusterSim = "0.45.2",
  data.table = "1.10.4",
  dada2 = "1.12",
  DESeq2 = "1.16.1",
  doParallel = "1.0.10",
  edgeR = "3.18.1",
  gridExtra = "2.2.1",
  ggplot2 = "2.2.1",
  ggtree = "1.12.0",
  igraph = "1.0.1",
  jsonlite = "1.5",
  magrittr = "1.5",
  markdown = "0.8",
  metagenomeSeq = "1.18.0",
  phyloseq = "1.28",
  pROC = "1.10.0",
  randomForest = "4.6",
  RColorBrewer = "1.1.2",
  rmarkdown = "1.6",
  ROCR = "1.0.7",
  scales = "0.4.1",
  ShortRead = "1.34.0",
  tidyverse = "1.2.1",
  UpSetR = "1.3")
################################################################################
# Check that the currently-installed version of R
# is at least the minimum required version.
################################################################################
R_min_version = "3.6.0"
R_version = paste0(R.Version()$major, ".", R.Version()$minor)
if(compareVersion(R_version, R_min_version) < 0){
  stop("You do not have the latest required version of R installed.\n", 
       "Launch should fail.\n",
       "See: http://cran.r-project.org/ and update your version of R.")
} else {
  message("R version looks okay:\n", R.version$major, ".", R.version$minor)
}
################################################################################
# Check install Bioconductor base
################################################################################
if(!requireNamespace("BiocManager", quietly = TRUE)){
  install.packages("BiocManager")
  library("BiocManager")
}
BiocManager::install(update = FALSE)

################################################################################
# Make sure devtools is installed...
################################################################################
if(!require("devtools")){
  install.packages("devtools")
}
if(
  compareVersion(a = as.character(packageVersion("devtools")),
                 b = "1.13.2") < 0
){
  message("\nUpdating devtools package to latest release version...\n")
  install.packages("devtools")
}
################################################################################
# Install basic required packages if not available/installed.
################################################################################
install_missing_packages = function(pkg, version = NULL, verbose = TRUE){
  availpacks = .packages(all.available = TRUE)
  # source("http://bioconductor.org/biocLite.R")
  require("BiocManager")
  
  # Check for missing packages
  missingPackage = FALSE
  if(!any(pkg %in% availpacks)){
    if(verbose){
      message("The following package is missing.\n",
              pkg, "\n",
              "Installation will be attempted...")
    }
    missingPackage <- TRUE
  }
  if(!is.null(version) & !missingPackage){
    # version provided and package not missing, so compare.
    if( compareVersion(a = as.character(packageVersion(pkg)),
                       b = version) < 0 ){
      if(verbose){
        message("Current version of package\n", 
                pkg, "\t", 
                packageVersion(pkg), "\n",
                "is less than required.
                Update will be attempted.")
      }
      missingPackage <- TRUE
      }
  }
  if(missingPackage){
    # Old
    # biocLite(pkg, suppressUpdates = TRUE)
    BiocManager::install(pkgs = pkg,  update = FALSE)
  }
}

################################################################################
# Loop on each package, check, install/update if needed
################################################################################
pkg1 = mapply(install_missing_packages,
              pkg = names(deppkgs), 
              version = deppkgs,
              MoreArgs = list(verbose = TRUE), 
              SIMPLIFY = FALSE,
              USE.NAMES = TRUE)

################################################################################
# Final installation of phyloseq suggested packages
################################################################################
# Old:
# biocLite("phyloseq", dependencies = "Suggests")

BiocManager::install(pkgs = "phyloseq", 
                     dependencies = "Suggests", 
                     type = "binary",
                     update = FALSE)
################################################################################
